@@ -6,7 +6,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -52,7 +52,7 @@ if (!$group_id)
 if (!user_isloggedin ())
   exit_permission_denied ();
 
-$is_admin = user_ismember ($group_id, MEMBER_FLAGS_ADMIN);
+$is_admin = user_is_group_admin ();
 $editable_reports = trackers_data_get_editable_reports ($group_id, $is_admin);
 $reports_updated = false;
 
@@ -505,7 +505,7 @@ function copy_tf_vals ($ff)
 
 function fixup_updated_vals ()
 {
-  global $cb_search_chk, $cb_report_chk, $cb_attr, $rank_extra;
+  global $cb_search_chk, $tf_search_val, $cb_attr, $rank_extra;
   $cb_search_chk = 0;
   $cb_attr['disabled'] = 'disabled';
   $rank_extra = " disabled='disabled'";
@@ -528,7 +528,7 @@ function show_mod_field ($field, $i, $fld = null)
   global $cb_report_chk, $cb_attr, $rank_extra;
   global $tf_report_val, $tf_colwidth_val;
   if (!report_data_field_is_shown ($field))
-    return;
+    return 0;
   extract (report_common_field_names ($field));
   if ($fld !== null)
     copy_field_vals ($fld, $field);
@@ -544,6 +544,7 @@ function show_mod_field ($field, $i, $fld = null)
       $cb_report_chk = 0; $tf_colwidth_val = '';
     }
   print_field ($i, $field);
+  return 1;
 }
 
 function rep_header_params ($group_id, $report_id)
@@ -570,7 +571,7 @@ function show_mod_report ($group_id, $title_arr, $report_id = 0)
   print_mod_report_header ($title, $params, $title_arr, $row);
   $i = 0;
   while ($field = trackers_list_all_fields ())
-    show_mod_field ($field, $i++, $fld);
+    $i += show_mod_field ($field, $i, $fld);
   print "</table>\n" . form_footer (false, 'submit');
   trackers_footer ();
   exit (0);
