@@ -7,7 +7,7 @@
 # Copyright (C) 2002-2006 Yves Perrin <yves.perrin--cern.ch>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -74,11 +74,7 @@ extract (sane_import ('post',
   ]
 ));
 form_check (['create_value', 'post_changes', 'submit']);
-
-if (!$group_id)
-  exit_no_group ();
-if (!user_ismember ($group_id, 'A'))
-  exit_permission_denied ();
+user_check_group_admin ();
 if (empty ($order_id))
   $order_id = 0;
 
