@@ -6,7 +6,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -85,7 +85,10 @@ if (!$group_id)
   exit_no_group();
 
 # Must be at least tracker admin or group admin.
-if (!member_check (0, $group_id, 2) && !user_ismember ($group_id, 'A'))
+if (
+  !member_check (0, $group_id, MEMBER_ROLE_TECHNAGER)
+    && !member_check ($group_id, MEMBER_FLAGS_ADMIN)
+)
   exit_permission_denied ();
 
 function fetch_preamble ($group_id, $artifact)
