@@ -7,7 +7,7 @@
 # Copyright (C) 2002-2006 Yves Perrin <yves.perrin--cern.ch>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -51,7 +51,7 @@ require_once ('../include/trackers/votes.php');
 if (!$group_id)
   exit_no_group ();
 
-$is_trackeradmin = member_check (0, $group_id, 2);
+$is_trackeradmin = member_check (0, $group_id, MEMBER_ROLE_TECHNAGER);
 
 function extract_reverse_order ()
 {
@@ -346,10 +346,16 @@ switch ($func)
 
     # To keep track of changes.
     $changes = [];
+    $deps_requested = false;
+    foreach (utils_get_dependable_trackers () as $t)
+      {
+        $var = "dependent_on_$t";
+        $deps_requested = $deps_requested || !empty ($$var);
+      }
 
     # Special case: we may be searching for an item, in that case
     # reprint the same page, plus search results.
-    if ($depends_search || $reassign_change_group_search)
+    if (($depends_search && !$deps_requested) || $reassign_change_group_search)
       {
         if ($depends_search)
           {
@@ -543,7 +549,7 @@ switch ($func)
     $spamscore = 1;
     if (member_check (0, $group_id))
       {
-        if (member_check (0, $group_id, 'A'))
+        if (member_check (0, $group_id, MEMBER_FLAGS_ADMIN))
           $spamscore = 5;
         else
           $spamscore = 3;
@@ -557,7 +563,7 @@ switch ($func)
 
   case 'unflagspam':
     # Unflag an alleged spam: for group admins only.
-    if (!member_check (0, $group_id, 'A'))
+    if (!member_check (0, $group_id, MEMBER_FLAGS_ADMIN))
       {
         # Do not use exit_not_logged_in(), because the user has no
         # valid reason to get here if he was not logged in in first place
