@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -339,30 +339,83 @@ function account_namevalid (
   return 0;
 }
 
-# Just check if the email address domain is not from a forbidden domain
-# or if it is not already associated to an email account.
-function account_emailvalid ($email)
+function account_email_malformed ($email)
 {
-  $res = db_execute ("SELECT user_id FROM user WHERE email LIKE ?", [$email]);
-  if (db_numrows ($res) > 0)
+  if (empty ($email))
     {
-      fb (
-        _("An account associated with that email address has already "
-          . "been created."),
-        1
-      );
-      return 0;
+      fb (_("You must supply a valid email address."), 1);
+      return 1;
     }
+  $alnum = 'a-zA-Z0-9';
+  # Compared to RFC-5322, atext doesn't include & and single quote.
+  $atext = $alnum
+    . '!#$%*+/=?^_`{|}~-'; # NB '-' should be the last character in the class.
+  $dot_atom_text = "[$atext][.$atext]*";
+  $preg = ',^' . "$dot_atom_text@$dot_atom_text" . '$,';
+
+  if (preg_match ($preg, $email))
+    return 0;
+  $msg = sprintf (
+    _("Supplied string *%s* doesn't look like an email address."), $email
+  );
+  fb ($msg, 1);
+  return 1;
+}
+
+function acount_email_placeholder ($email)
+{
+  if ($email !== USER_EMAIL_PLACEHOLDER)
+    return 0;
+  $msg = sprintf (
+    _("The email address *%s* is reserved for deleted accounts."), $email
+  );
+  fb ($msg, 1);
+  return 1;
+}
+
+function account_email_registered ($email)
+{
+  # Never mind deleted accounts and squads;
+  # only look for active and pending accounts.
+  $status = [USER_STATUS_ACTIVE, USER_STATUS_PENDING];
+  $res = db_execute (
+    "SELECT user_id FROM user WHERE email = ? AND status "
+    . utils_in_placeholders ($status),
+    array_merge ([$email], $status)
+  );
+  if (!db_numrows ($res))
+    return 0;
+  $msg = sprintf (
+    _("An account with the email *%s* has already been created."), $email
+  );
+  fb ($msg, 1);
+  return 1;
+}
+
+function account_regexforbidden ($email)
+{
+  # Despite its name, $forbid_mail_domains_regexp is matched against
+  # the whole email rather than its domain.  The feature isn't used
+  # as of 2025-03, actually.
   utils_get_content ("forbidden_mail_domains");
   if (empty ($GLOBALS['forbid_mail_domains_regexp']))
-    return 1;
+    return 0;
   if (!preg_match ($GLOBALS['forbid_mail_domains_regexp'], $email))
-    return 1;
-  fb (
-    _("It is not allowed to associate an account with this email address."),
-    1
-  );
-  return 0;
+    return 0;
+  $m = _("It is not allowed to associate an account with this email address.");
+  fb ($m, 1);
+  return 1;
+}
+
+function account_emailvalid ($email)
+{
+  if (account_email_malformed ($email))
+    return 0;
+  if (acount_email_placeholder ($email))
+    return 0;
+  if (account_email_registered ($email))
+    return 0;
+  return !account_regexforbidden ($email);
 }
 
 function account_groupnamevalid ($name)
