@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -51,6 +51,7 @@ function exit_error ($title = '', $text = null, $status = false)
   global $HTML;
 
   exit_header ($status);
+  init_ensure_constants ();
   $msg = $title;
   if ($msg === null)
     $msg = $text;
@@ -147,4 +148,13 @@ function exit_test_usesmail ($group_id)
   if (!$project->Uses ('mail'))
     exit_error (_("Error"), _("This project has turned off mailing lists"));
 }
+
+function exit_user_not_found ($user)
+{
+  if (ctype_digit ($user))
+    $msg = _("User #%s not found.");
+  else
+    $msg = _("User *%s* not found.");
+  exit_error (markup_rich (sprintf ($msg, $user)));
+}
 ?>
