@@ -7,7 +7,7 @@
 # Copyright (C) 2008 Aleix Conchillo Flaque
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -922,7 +922,7 @@ function group_set_preference_args ($name, $val)
 # * $preference_name is a key-to-value array and $value is null.
 function group_set_preference ($group_id, $preference_name, $value = null)
 {
-  if (!user_ismember ($group_id, 'A'))
+  if (!user_is_group_admin ($group_id))
     return false;
 
   $prefs = group_set_preference_args ($preference_name, $value);
