@@ -7,7 +7,7 @@
 # Copyright (C) 2008 Aleix Conchillo Flaque
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -163,49 +163,150 @@ function html_image_dir ($theme, $suffix = null)
   return $ret;
 }
 
-function html_nextprev_link ($search_url, $varprefix, $offset)
+function html_nextprev_separator ()
 {
-  global $max_rows;
-  print "<a href=\"$search_url&amp;{$varprefix}offset="
-    . "$offset&amp;{$varprefix}max_rows="
-    . utils_specialchars ($max_rows) . "#{$varprefix}results\">";
+  return '&nbsp; &nbsp; &nbsp;';
 }
 
-function html_nextprev ($search_url, $rows, $rows_returned, $varprefix = false)
+function html_nextprev_link ($url, $offset, $max_rows)
 {
-  global $offset, $sys_home, $max_rows;
+  $start = $offset + 1;
+  return "<a href=\"$url&amp;start=$start&amp;max_rows=$max_rows#results\">";
+}
 
-  if (!$varprefix)
-    $varprefix = '';
-  else
-    $varprefix .= '_';
+function html_medium_link ($url, $offset, $max_rows)
+{
+  $link = html_nextprev_link ($url, $offset, $max_rows);
+  # TRANSLATORS: The string shows a range of item numbers to display.
+  $msg = sprintf (_('%1$s&ndash;%2$s'), $offset + 1, $offset + $max_rows);
+  return "$link$msg</a>" . html_nextprev_separator ();
+}
 
-  if ($rows_returned < $rows && !$offset)
-    return;
-  $prev_msg = _("Previous Results");
-  $next_msg = _("Next Results");
-  print "\n<br /><p class=\"nextprev\">\n";
+function html_less ($url, $offset, $max_rows)
+{
+  $page_no = $offset / $max_rows;
+  if ($page_no < 3)
+    return '';
+  $page_no = intval ($page_no / 2);
+  return html_medium_link ($url, $page_no * $max_rows, $max_rows);
+}
 
-  if ($offset)
+function html_more ($url, $offset, $max_rows, $total_rows)
+{
+  $where_end_starts = intval ($total_rows / $max_rows) * $max_rows;
+  $offset += $max_rows * 2;
+  $middle_pages = ($where_end_starts - $offset) / $max_rows;
+  if ($middle_pages < 1)
+    return '';
+  if ($middle_pages >= 2)
     {
-      $prev_offset = $offset - $max_rows;
-      if ($prev_offset < 0)
-        $prev_offset = 0;
-      html_nextprev_link ($search_url, $varprefix, $prev_offset);
-      print html_image ("arrows/previous.png") . " $prev_msg</a>";
+      $offset = ceil ($offset / $max_rows) * $max_rows;
+      $middle_pages = ($where_end_starts - $offset) / $max_rows;
     }
-  else
-    print html_image ("arrows/previousgrey.png") . " <i>$prev_msg</i>";
-  print "&nbsp; &nbsp; &nbsp;";
+  $offset += intval (($middle_pages - 1) / 2) * $max_rows;
+  return html_medium_link ($url, $offset, $max_rows);
+}
 
-  if ($rows_returned > $rows)
+function html_prev ($url, $offset, $max_rows, $total_rows)
+{
+  if ($total_rows <= $max_rows)
+    return '';
+  $begin_msg = _("Begin"); $prev_msg = _("Previous");
+  $sep = html_nextprev_separator ();
+  if (!$offset)
+    return html_image ('arrows/firstgrey.png') . " <i>$begin_msg</i>$sep"
+      . html_image ("arrows/previousgrey.png") . " <i>$prev_msg</i>";
+  $ret = html_nextprev_link ($url, 0, $max_rows);
+  $ret .= html_image ('arrows/first.png') . " $begin_msg</a>$sep";
+  $ret .= html_less ($url, $offset, $max_rows);
+  $offset -= $max_rows;
+  if ($offset < 0)
+    $offset = 0;
+  $ret .= html_nextprev_link ($url, $offset, $max_rows);
+  return $ret . html_image ('arrows/previous.png') . " $prev_msg</a>$sep";
+}
+
+function html_next ($url, $offset, $max_rows, $total_rows)
+{
+  if ($total_rows <= $max_rows)
+    return '';
+  $next_msg = _("Next"); $end_msg = _("End");
+  $sep = html_nextprev_separator ();
+  $rows = min ($max_rows, $total_rows - $offset);
+  if ($offset + $max_rows >= $total_rows)
+    return "$sep<i>$next_msg</i> " . html_image ("arrows/nextgrey.png")
+      . "$sep<i>$end_msg</i> " . html_image ('arrows/lastgrey.png');
+  $ret = $sep . html_nextprev_link ($url, $offset + $rows, $max_rows);
+  $ret .= "$next_msg " . html_image ("arrows/next.png") . "</a>$sep";
+  $ret .= html_more ($url, $offset, $max_rows, $total_rows);
+  $last_page = $total_rows - ($total_rows % $max_rows);
+  if ($last_page == $total_rows)
+    $last_page -= $max_rows;
+  $ret .= html_nextprev_link ($url, $last_page, $max_rows);
+  return "$ret$end_msg " . html_image ("arrows/last.png") . "</a>";
+}
+
+function html_nextprev_extract_params ($default_max_rows = 50)
+{
+  extract (sane_import ('request',
+    [
+      'digits' => [
+        ['max_rows', [1, 4913]],
+        ['offset', [0, 410338672]], ['start', [1, 410338673]]
+      ]
+    ]
+  ));
+  if (empty ($max_rows)
+    || !intval ($max_rows) # Values like '00' aren't empty, but intval is zero.
+  )
+    $max_rows = $default_max_rows;
+  if (empty ($offset))
+    $offset = 0;
+  if (!empty ($start))
+    $offset = $start - 1;
+  foreach (['max_rows', 'offset'] as $v)
+    $GLOBALS[$v] = intval ($$v);
+}
+
+function html_nextprev_item_count ($offset, $max_rows, $total_rows, $have_form)
+{
+  $first_item = $offset;
+  if ($have_form)
+    $first_item = form_hidden (['max_rows' => $max_rows])
+      . form_input ('text', 'start', $offset + 1, 'size="3"');
+  $latest_item = min ($offset + $max_rows, $total_rows);
+  return sprintf (
+    # TRANSLATORS: The first argument is the number of the first item shown,
+    # the second argument is the number of the last item shown,
+    # the third argument is the total number of items.
+    _('%1$s&ndash;%2$s / %3$s'), $first_item, $latest_item, $total_rows
+  );
+}
+
+function html_nextprev_str ($url, $offset, $max_rows, $total_rows)
+{
+  $ret = "<p class=\"nextprev\">\n";
+  if ($total_rows <= $max_rows)
     {
-      html_nextprev_link ($search_url, $varprefix, $offset + $rows);
-      print "$next_msg " . html_image ("arrows/next.png") . "</a>";
+      $msg = ngettext ( "%d matching item", "%d matching items", $total_rows);
+      return $ret . sprintf ($msg, $total_rows);
     }
-  else
-    print "<i>$next_msg</i> " . html_image ("arrows/nextgrey.png");
-  print "</p>\n";
+  $have_form = $total_rows > $max_rows;
+  if ($have_form)
+    $ret = form_tag (['action' => "$url#results"]) . $ret;
+  $ret .= html_prev ($url, $offset, $max_rows, $total_rows);
+  $ret .= html_nextprev_item_count (
+    $offset, $max_rows, $total_rows, $have_form
+  );
+  $ret .= html_next ($url, $offset, $max_rows, $total_rows) . "</p>\n";
+  if ($have_form)
+    $ret .= "</form>\n";
+  return $ret;
+}
+
+function html_nextprev ($url, $offset, $max_rows, $total_rows)
+{
+  print html_nextprev_str ($url, $offset, $max_rows, $total_rows);
 }
 
 function html_anchor ($content, $name)
@@ -878,7 +979,7 @@ function site_user_footer ($params)
 # Administrative page functions.
 function site_admin_header ($params)
 {
-  session_require (['group' => '1','admin_flags' => 'A']);
+  session_require (['group' => SESSION_ADMIN_GROUP]);
   html_header ($params);
 }
 
