@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -47,7 +47,14 @@ define ('MEMBER_FLAGS_ADMIN', 'A');
 define ('MEMBER_FLAGS_PENDING', 'P');
 define ('MEMBER_FLAGS_SQUAD', 'SQD');
 
+define ('MEMBER_ROLE_TECHNICIAN', '1');
+define ('MEMBER_ROLE_TECHNAGER', '2');
+define ('MEMBER_ROLE_MANAGER', '3');
+
 $MEMBER_FLAGS_ACTIVE = [MEMBER_FLAGS_MEMBER => 1, MEMBER_FLAGS_ADMIN => 1];
+$member_roles = [
+  MEMBER_ROLE_TECHNICIAN, MEMBER_ROLE_TECHNAGER, MEMBER_ROLE_MANAGER
+];
 
 function member_history_label_on_add ($status)
 {
@@ -68,8 +75,10 @@ function member_assign_uidNumber ($user_id)
   member_update_file ($user_id, 'passwd');
   db_execute ("
     UPDATE `user_group` u, `groups` g
-    SET u.`cache_uidNumber` = ?, u.`cache_gidNumber` = g.`gidNumber`
-    WHERE `user_id` = ? AND u.`group_id` = g.`group_id`", [$uid, $user_id]
+    SET u.`cache_uidNumber` = ?, u.`cache_gidNumber` = g.`gidNumber`,
+      u.`cache_user_name` = ?
+    WHERE `user_id` = ? AND u.`group_id` = g.`group_id`",
+    [$uid, user_getname ($user_id), $user_id]
   );
 }
 
@@ -79,7 +88,7 @@ function member_user_group_fields ($user_id, $group_id, $status)
   $gid = group_get_gidNumber ($group_id);
   $cols = [
    'admin_flags' => $status, 'cache_gidNumber' => $gid,
-   'cache_uidNumber' => $uid
+   'cache_uidNumber' => $uid, 'cache_user_name' => user_getname ($user_id)
   ];
   $cond = ['user_id' => $user_id, 'group_id' => $group_id];
   return [$cols, $cond];
@@ -216,8 +225,14 @@ function member_approve ($user_id, $group_id)
   member_assign_uidNumber ($user_id);
   list ($cols, $cond) =
     member_user_group_fields ($user_id, $group_id, MEMBER_FLAGS_MEMBER);
+  $where = $cond_list = [];
+  foreach ($cond as $k => $v)
+    {
+      $cond_list[] = $v;
+      $where[] = "$k = ?";
+    }
   $result = db_autoexecute (
-    'user_group', $cols, DB_AUTOQUERY_UPDATE, join (' AND ', $cond)
+    'user_group', $cols, DB_AUTOQUERY_UPDATE, join (' AND ', $where), $cond_list
   );
   if (!$result)
     return $result;
@@ -399,15 +414,11 @@ function member_check_single_perm ($value, $flag_level, $strict)
     return true; # If the value is equal to the flag, $u is obviously included.
   if ($strict)
     return false;
-  if (2 == $value && in_array ($flag_level, [1, 3]))
-    # The value is equal to 2 (manager and tech) if tech (1)
-    # or manager (3) is asked.
+  if (MEMBER_ROLE_TECHNAGER == $value
+      && in_array ($flag_level, [MEMBER_ROLE_TECHNICIAN, MEMBER_ROLE_MANAGER]))
     return true;
-  if (2 == $flag_level  && in_array ($value, [1, 3]))
-    # If the value is equal to 3 (manager) or 1 (techn) if tech
-    # and manager (2) is asked, it is "true".
-    return true;
-  return false;
+  return MEMBER_ROLE_TECHNAGER == $flag_level
+    && in_array ($value, [MEMBER_ROLE_TECHNICIAN, MEMBER_ROLE_MANAGER]);
 }
 
 function member_check_array_perms ($group_id, $flag, $uids, $strict)
@@ -461,25 +472,25 @@ function member_users_are_in_group ($group_id, $uids)
   return $ret;
 }
 
-# Check membership: by default, check only if someone is member of a project.
+# Check membership: by default, check only if someone is member of a group.
 #
 # With the flag option, you can check for specific right:
 #    - the first letter of the flag should designate the tracker
-#       (B = bugs, P = patch...
-#        please use member_create_tracker_flag(ARTIFACT))
+#       (B = bugs, P = patch...)
 #    - the second letter, if specified, designate a role
 #       1 = technician
 #       2 = technician AND manager
 #       3 = manager
 #
 # The strict variable permit to have a return "true" only if the flag
-# found is exactly equal to the flag asked. For instance, if you are
-# looking for someone who is only technician, and not techn. and manager,
+# found is exactly equal to the flag asked.  For instance, if you are
+# looking for someone who is only technician, and not techn and manager,
 # you can use that flag.
 function member_check_array ($user_id, $group_id, $flag = 0, $strict = 0)
 {
-  if (in_array ($flag, [1, 2, 3]))
-    $flag = member_create_tracker_flag (ARTIFACT) . "$flag";
+  global $member_roles;
+  if (in_array ($flag, $member_roles))
+    $flag = member_create_tracker_flag (ARTIFACT) . $flag;
   list ($uids, $ret) = member_check_propagate_uids ($user_id);
   if (empty ($uids))
     return $ret;
