@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -298,6 +298,7 @@ function sane_func ($x)
   global $sane_sanitizers;
   if (isset($sane_sanitizers[$x]))
     return $sane_sanitizers[$x];
+  trigger_error ('No [' . error_print_r ($x) . '] sanitizer found');
   return null;
 }
 
@@ -312,6 +313,8 @@ function sane_assign_arr_func ($arg_in, &$func_arg)
       $ret = $arg_in[0];
       $func_arg = $arg_in[1];
     }
+  if ($ret === null)
+    return null;
   return sane_func ($ret);
 }
 
