@@ -6,7 +6,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -76,6 +76,8 @@
 # http://www.ietf.org/rfc/rfc2965.txt (status: proposed standard)
 # https://savannah.gnu.org/task/?6800 (don't use a leading dot)
 
+define ('SESSION_ADMIN_GROUP', 1); # Actually no such group exists.
+
 foreach (['sane', 'account', 'random-bytes'] as $h)
   require_once (dirname (__FILE__) . "/$h.php");
 
@@ -252,6 +254,8 @@ function session_setglobals ($user_id)
 
 function session_hash_parts ($hash)
 {
+  if (empty ($hash))
+    return ['', null];
   if (preg_match ('/(.*;)(.*)/', $hash, $m))
     return [$m[2], $m[1]];
   return [$hash, null];
