@@ -9,7 +9,7 @@
 # Copyright (C) 2002-2006 Yves Perrin <yves.perrin--cern.ch>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -177,7 +177,7 @@ function trackers_data_show_notification_settings ($group_id, $tracker)
   $grtrsettings = trackers_data_get_notification_settings (
     $group_id, $tracker
   );
-  if (!(user_ismember ($group_id, MEMBER_FLAGS_ADMIN)))
+  if (!user_is_group_admin ($group_id))
     return;
   $cat_ck = $glob_ck = $both_ck = false;
   if ($grtrsettings['glnotif'] == 0)
@@ -820,6 +820,39 @@ function trackers_data_is_value_set_empty (
   return db_numrows ($res) <= 0;
 }
 
+function trackers_data_copy_default_val ($field_id, $group_id, $row)
+{
+  $arr = [];
+  foreach (['value_id', 'value', 'description', 'order_id', 'status',
+    'email_ad'] as $k)
+    $arr[$k] = $row[$k];
+  $arr['bug_field_id'] = $field_id;
+  $arr['group_id'] = $group_id;
+  $res = db_autoexecute (ARTIFACT . "_field_value", $arr, DB_AUTOQUERY_INSERT);
+  if (db_affected_rows ($res))
+    return;
+  fb (_("Insert of default value failed."), 0);
+  db_error ();
+}
+
+function trackers_data_delete_field_value ($field_id, $group_id)
+{
+  $table = ARTIFACT . '_field_value';
+  db_execute (
+    "DELETE FROM `$table` WHERE `bug_field_id` = ? AND `group_id` = ?",
+    [$field_id, $group_id]
+  );
+}
+
+function trackers_data_get_default_field_values ($field_id)
+{
+  return db_execute ("
+    SELECT value_id, value, description, order_id, status, email_ad
+    FROM " . ARTIFACT . "_field_value WHERE bug_field_id = ? AND group_id = ?",
+    [$field_id, GROUP_NONE]
+  );
+}
+
 # Initialize the set of values for a given field for a given group by using
 # the system default (default values belong to group_id  = GROUP_NONE).
 function trackers_data_copy_default_values (
@@ -833,49 +866,12 @@ function trackers_data_copy_default_values (
   # because default values belong to GROUP_NONE by definition.
   if ($group_id == GROUP_NONE)
     return;
-  # First delete the existing value if any.
-  $res = db_execute ("
-    DELETE FROM " . ARTIFACT . "_field_value
-    WHERE bug_field_id = ? AND group_id = ?",
-    [$field_id, $group_id]
-  );
-
-  # Second insert default values (if any) from group 'None'.
-  # Rk: The target table of the INSERT statement cannot appear in
-  # the FROM clause of the SELECT part of the query, because it's forbidden
-  # in ANSI SQL to SELECT. So do it by hand !
-  $res = db_execute ("
-    SELECT value_id, value, description, order_id, status
-    FROM " . ARTIFACT . "_field_value WHERE bug_field_id = ? AND group_id = ?",
-    [$field_id, GROUP_NONE]
-  );
-  $rows = db_numrows ($res);
-
-  for ($i = 0; $i < $rows; $i++)
-    {
-      $value_id = db_result ($res, $i, 'value_id');
-      $value = db_result ($res, $i, 'value');
-      $description = db_result ($res, $i, 'description');
-      $order_id = db_result ($res, $i, 'order_id');
-      $status  = db_result ($res, $i, 'status');
-
-      $res_insert = db_autoexecute (
-        ARTIFACT . "_field_value",
-        [
-          'bug_field_id' => $field_id, 'group_id' => $group_id,
-          'value_id' => $value_id, 'value' => $value,
-          'description' => $description, 'order_id' => $order_id,
-          'status' => $status
-        ],
-        DB_AUTOQUERY_INSERT
-      );
-
-      if (db_affected_rows ($res_insert) < 1)
-        {
-          fb (_("Insert of default value failed."), 0);
-          db_error ();
-        }
-    }
+  trackers_data_delete_field_value ($field_id, $group_id);
+  # ANSI SQL disallows the target table of the INSERT statement appear
+  # in the FROM clause of the SELECT part of the query, so do it by hand.
+  $res = trackers_data_get_default_field_values ($field_id);
+  while ($row = db_fetch_array ($res))
+    trackers_data_copy_default_val ($field_id, $group_id, $row);
 }
 
 function trackers_data_get_cached_field_value ($field, $group_id, $value_id)
@@ -965,22 +961,11 @@ function trackers_data_create_value (
     fb (_("New field value inserted."));
 }
 
-# Insert a new value for a given field for a given group.
-function trackers_data_update_value (
-  $item_fv_id, $field, $group_id, $value, $description, $order_id,
-  $status = 'A'
-)
+function trackers_data_update_value_params ($item_fv_id, $field, $group_id)
 {
-  if (preg_match ("/^\s*$/", $value))
-    {
-      fb (_("Empty field value not allowed"), 0);
-      return;
-    }
-
   # Updating a bug field value that belong to GROUP_NONE is
   # forbidden. These are default values that cannot be changed, so
   # make sure to copy the default values first in the group context first.
-
   if ($res = trackers_data_is_default_value ($item_fv_id))
     {
       trackers_data_copy_default_values ($field, $group_id);
@@ -996,24 +981,42 @@ function trackers_data_update_value (
       $where_cond = "bug_fv_id = ? AND group_id <> ?";
       $where_cond_params = [$item_fv_id, GROUP_NONE];
     }
+  return [$where_cond, $where_cond_params];
+}
 
-  $result = db_autoexecute (
-    ARTIFACT . "_field_value",
-    [
-     'value' => $value,
-     'description' => $description,
-     'order_id' => $order_id,
-     'status' => $status
-    ],
-    DB_AUTOQUERY_UPDATE, "$where_cond", $where_cond_params
-  );
-
+function trackers_data_report_update_value ($result)
+{
   if (db_affected_rows ($result) < 1)
     fb (_("Update of field value failed."), 1);
   else
     fb (_("New field value updated."));
 }
 
+# Insert a new value for a given field for a given group.
+function trackers_data_update_value (
+  $item_fv_id, $field, $group_id, $value, $description, $order_id,
+  $status = 'A'
+)
+{
+  if (preg_match ("/^\s*$/", $value))
+    {
+      fb (_("Empty field value not allowed"), 0);
+      return;
+    }
+  list ($where_cond, $where_cond_params) =
+    trackers_data_update_value_params ($item_fv_id, $field, $group_id);
+
+  $result = db_autoexecute (
+    ARTIFACT . "_field_value",
+    [
+     'value' => $value, 'description' => $description,
+     'order_id' => $order_id, 'status' => $status, 'email_ad' => ''
+    ],
+    DB_AUTOQUERY_UPDATE, "$where_cond", $where_cond_params
+  );
+  trackers_data_report_update_value ($result);
+}
+
 # Reset a field settings to its defaults usage (values are untouched). The defaults
 # always belong to group_id GROUP_NONE so make sure we don't delete entries for
 # GROUP_NONE.
@@ -1558,20 +1561,13 @@ function update_close_date ($item_id, $vfl, $row, &$upd_list)
   trackers_data_add_history ('close_date', $row['close_date'], $now, $item_id);
 }
 
-function handle_update (
-  $group_id, $item_id, $dependent_on_task, $dependent_on_bugs,
-  $dependent_on_support, $dependent_on_patch, $canned_response, $vfl,
-  &$changes, &$extra_addresses
+function update_field_vals (
+  $row, $field_transition, $vfl, &$changes, &$extra_addr
 )
 {
-  list ($row, $field_transition) = fetch_data ($group_id, $item_id, $vfl);
-  if ($row === false)
-    return;
-  $change_exists = 0;
-
-  $field_transition_accepted = $upd_list = $extra_addr = [];
-  if (trim ($extra_addresses) != '')
-    $extra_addr[] = $extra_addresses;
+  $item_id = $row['bug_id'];
+  $group_id = $row['group_id'];
+  $field_transition_accepted = $upd_list = [];
   foreach ($vfl as $field => $value)
     {
       if (transition_not_needed ($field, $row))
@@ -1596,6 +1592,38 @@ function handle_update (
         $field, $group_id, $old_value, $value, $changes
       );
     } # foreach ($vfl as $field => $value)
+  return [$field_transition_accepted, $upd_list];
+}
+
+function update_comment ($row, $vfl, $canned_response, &$changes)
+{
+  $item_id = $row['bug_id'];
+  $group_id = $row['group_id'];
+  $details = '';
+  if (array_key_exists ('comment', $vfl))
+    $details = $vfl['comment'];
+
+  $details = trackers_data_append_canned_response ($details, $canned_response);
+  return update_details ($details, $item_id, $group_id, $vfl, $changes);
+}
+
+function handle_update (
+  $group_id, $item_id, $dependent_on_task, $dependent_on_bugs,
+  $dependent_on_support, $dependent_on_patch, $canned_response, $vfl,
+  &$changes, &$extra_addresses
+)
+{
+  list ($row, $field_transition) = fetch_data ($group_id, $item_id, $vfl);
+  if ($row === false)
+    return;
+
+  $extra_addr = [];
+  if (trim ($extra_addresses) != '')
+    $extra_addr[] = $extra_addresses;
+  list ($field_transition_accepted, $upd_list) =
+    update_field_vals (
+      $row, $field_transition, $vfl, $changes, $extra_addr
+    );
 
   # Now we run transitions other fields update. This function does check
   # what already changed and that we shan't automatically update.
@@ -1603,24 +1631,14 @@ function handle_update (
     $item_id, $field_transition_accepted, $changes
   );
 
-  # Comments field history is handled a little differently. Followup comments
-  # are added in the bug history along with the comment type.
-  # Comments are called 'details' here for historical reason.
-  $details = trackers_data_append_canned_response (
-    $vfl['comment'], $canned_response
-  );
-
-  $change_exists |= update_details (
-    $details, $item_id, $group_id, $vfl, $changes
-  );
-
+  $change_exists = update_comment ($row, $vfl, $canned_response, $changes);
   $change_exists |= update_cookbook (
     $item_id, $group_id, $row, $vfl, $changes, $upd_list
   );
   update_close_date ($item_id, $vfl, $row, $upd_list);
 
   # Enter new dependencies.
-  $artifacts = array_diff (utils_get_tracker_list (), ['cookbook']);
+  $artifacts = utils_get_dependable_trackers ();
   $address = '';
   foreach ($artifacts as $art)
     {
@@ -1667,71 +1685,48 @@ function trackers_data_handle_update (
   );
 }
 
-function trackers_data_reassign_item (
-  $item_id, $reassign_change_group, $reassign_change_artifact
-)
+function trackers_data_reassign_get_new_group_id ($change_group, $new_artifact)
 {
   global $group_id;
 
-  # Can only be done by a tracker manager.
-  if (!member_check (0, $group_id, 2))
-    return false;
-  # If the new group_id is equal to the current one, nothing need
-  # to be done, unless the artifact changed.
-  # If the new group_id does not exists, nothing to be done either,
-  # unless the artifact changed: if no new valid group_id, let
-  # consider that it does not require a change.
-  $new_group_id = group_getid ($reassign_change_group);
+  $new_group_id = null;
+  if ($change_group)
+    $new_group_id = group_getid ($change_group);
   if (!$new_group_id)
     $new_group_id = $group_id;
-
-  if ($new_group_id == $group_id && ARTIFACT == $reassign_change_artifact)
+  if ($new_group_id == $group_id && ARTIFACT == $new_artifact)
     {
       fb (_("No reassignation required or possible."), 1);
-      return false;
+      return NULL;
     }
+  if ($new_artifact)
+    return $new_group_id;
+  fb (_("Unable to find out to which artifact the item is to be\n"
+        . "reassigned, exiting."), 1);
+  return NULL;
+}
 
-  $now = time ();
-
-  # To reassign an item, we close the item and we reopen a new one
-  # at the appropriate place, copying information from the previous one
-  # We do this because trackers may have specific fields not compatible
-  # each others. Simply erase previous information could cause data loss.
-
-  # Fetch all the information.
+function trackers_data_fetch_item ($item_id)
+{
   $res_data = db_execute (
     "SELECT * FROM " . ARTIFACT . " WHERE bug_id = ?",
     [$item_id]
   );
-  $row_data = db_fetch_array ($res_data);
+  return db_fetch_array ($res_data);
+}
 
-  # Duplicate the report.
-  if (!$reassign_change_group)
-    $reassign_change_group = $group_id;
+function trackers_data_copy_item (
+  $new_artifact, $new_group_id, $data, $now
+)
+{
+  $fields = ['group_id' => $new_group_id, 'status_id' => 1, 'date' => $now];
+  foreach (['severity', 'submitted_by', 'summary', 'details', 'priority',
+    'planned_starting_date', 'planned_close_date' , 'percent_complete' ,
+    'originator_email'] as $k
+  )
+    $fields[$k] = $data[$k];
 
-  if (!$reassign_change_artifact)
-    {
-      fb (_("Unable to find out to which artifact the item is to be\n"
-            . "reassigned, exiting."), 1);
-          return false;
-    }
-
-  # Move item.
-  $result = db_autoexecute (
-    $reassign_change_artifact,
-    [
-      'group_id' => $new_group_id, 'status_id' => 1, 'date' => $now,
-      'severity' => $row_data['severity'],
-      'submitted_by' => $row_data['submitted_by'],
-      'summary' => $row_data['summary'], 'details' => $row_data['details'],
-      'priority' => $row_data['priority'],
-      'planned_starting_date' => $row_data['planned_starting_date'],
-      'planned_close_date' => $row_data['planned_close_date'],
-      'percent_complete' => $row_data['percent_complete'],
-      'originator_email' => $row_data['originator_email']
-    ],
-    DB_AUTOQUERY_INSERT
-  );
+  $result = db_autoexecute ($new_artifact, $fields, DB_AUTOQUERY_INSERT);
 
   if (!$result)
     {
@@ -1739,19 +1734,44 @@ function trackers_data_reassign_item (
       return false;
     }
   fb (_("New item created."));
-
-  # Need to get the new item value.
   $new_item_id =  db_insertid ($result);
-  if (!$new_item_id)
+  if ($new_item_id)
+    return $new_item_id;
+  fb (_("Unable to find the ID of the new item."), 1);
+  return false;
+}
+
+function trackers_data_reassign_copy_comments (
+  $item_id, $new_item_id, $new_artifact
+)
+{
+  $res = db_execute (
+    "SELECT * FROM " . ARTIFACT . "_history WHERE bug_id = ? AND type = 100",
+    [$item_id]
+  );
+  $table = $new_artifact . "_history";
+  while ($row = db_fetch_array ($res, DB_FETCH_ASSOC))
     {
-      fb (_("Unable to find the ID of the new item."), 1);
-      return false;
+      foreach (['bug_history_id', 'new_value', 'spamscore', 'ip'] as $k)
+        unset ($row[$k]);
+      $row['bug_id'] = $new_item_id;
+      $result = db_autoexecute ($table, $row, DB_AUTOQUERY_INSERT);
+      if ($result)
+        continue;
+      fb (_("Unable to duplicate a comment from the original item."), 1);
     }
+}
+
+function trackers_data_reassign_add_history (
+  $item_id, $new_item_id, $new_group_id, $new_artifact
+)
+{
+  global $group_id;
 
   $grp_item = group_getname ($group_id) . ', '
     . utils_get_tracker_prefix (ARTIFACT) . " #$item_id";
   $new_grp_item = group_getname ($new_group_id) . ', '
-    . utils_get_tracker_prefix ($reassign_change_artifact) . " #$new_item_id";
+    . utils_get_tracker_prefix ($new_artifact) . " #$new_item_id";
 
   trackers_data_add_history (
     'Reassign Item', $grp_item, $new_grp_item, $item_id, false, ARTIFACT, 1
@@ -1759,173 +1779,190 @@ function trackers_data_reassign_item (
 
   trackers_data_add_history (
     'Reassign item', $grp_item, $new_grp_item,
-    $new_item_id, false, $reassign_change_artifact, 1
+    $new_item_id, false, $new_artifact, 1
   );
+  trackers_data_reassign_copy_comments ($item_id, $new_item_id, $new_artifact);
+}
 
-  # Duplicate the comments.
-  $res_history = db_execute (
-    "SELECT * FROM " . ARTIFACT . "_history WHERE bug_id = ? AND type = 100",
-    [$item_id]
-  );
-  while ($row_history = db_fetch_array ($res_history))
-    {
-      $result = db_autoexecute (
-        $reassign_change_artifact . "_history",
-        [
-          'bug_id' => $new_item_id, 'field_name' => $row_history['field_name'],
-          'old_value' => $row_history['old_value'],
-          'mod_by' => $row_history['mod_by'], 'date' => $row_history['date'],
-          'type' => $row_history['type']
-        ],
-        DB_AUTOQUERY_INSERT
-      );
-      if (!$result)
-        {
-          fb (
-            _("Unable to duplicate a comment from the original item\n"
-              . "report information."),
-            1);
-        }
-    }
+function trackers_data_get_artifact_columns ()
+{
+  $res = db_execute ("SHOW COLUMNS FROM " . ARTIFACT);
+  $list = [];
+  while ($row = db_fetch_array ($res))
+    $list[] = $row['Field'];
+  return $list;
+}
 
-  # Add a comment giving every original information.
+function trackers_data_reassign_compose_comment ($item_id, $row_data)
+{
+  global $group_id;
   $comment = "This item has been reassigned from the group "
     . group_getname ($row_data['group_id']) . " " . ARTIFACT
     . " tracker to your tracker.\n\nThe original report is still available at "
     . ARTIFACT . " #$item_id\n\n"
     . "Following are the information included in the original report:\n\n";
 
-  $res_show = db_execute ("SHOW COLUMNS FROM " . ARTIFACT);
-  $list = array();
-  while ($row_show = db_fetch_array ($res_show))
+  foreach (trackers_data_get_artifact_columns () as $l => $v)
     {
-      # Build a list of any possible field.
-      $list[] = $row_show['Field'];
-    }
-
-  foreach ($list as $l => $v)
-    {
-      if ($row_data[$l])
-        {
-          $comment .= "[field #" . $l . "] ";
-          $comment .= trackers_field_display (
-            $v, $group_id, $row_data[$l], false, true, true, true
-          );
-          $comment .= "<br />\n";
-        }
-    }
-
-  $result = db_autoexecute (
-    $reassign_change_artifact . "_history",
-    [
-      'bug_id' => $new_item_id, 'field_name' => 'details',
-      'old_value' => $comment, 'mod_by' => user_getid (),
-      'date' => $now, 'type' => 100
-    ],
-    DB_AUTOQUERY_INSERT
-  );
-
-  if (!$result)
-    {
-      fb (
-        _("Unable to add a comment with the original item report\n"
-          . "information."),
-        1
+      if (!$row_data[$l])
+        continue;
+      $comment .= "[field #$l] ";
+      $comment .= trackers_field_display (
+        $v, $group_id, $row_data[$l], false, true, true, true
       );
+      $comment .= "<br />\n";
     }
+  return $comment;
+}
 
-  # Usually, reassigning means duplicating data.
-  # In case of attached files, we simply reassign the file to another
-  # item. This could avoid wasting too much disk space as file are expected
-  # to be much bigger than CC list and alike.
+function trackers_data_reassign_add_comment (
+  $new_item_id, $new_artifact, $comment, $now
+)
+{
+  $table = $new_artifact . "_history";
+  $fields = [
+    'bug_id' => $new_item_id, 'field_name' => 'details', 'date' => $now,
+    'old_value' => $comment, 'mod_by' => user_getid (), 'type' => 100
+  ];
+  if (db_autoexecute ($table, $fields, DB_AUTOQUERY_INSERT))
+    return;
+  fb (
+    _("Unable to add a comment with the original item report\ninformation."), 1
+  );
+}
+
+function trackers_data_reassign_files ($item_id, $new_item_id, $new_artifact)
+{
   $result = db_autoexecute (
-    "trackers_file",
-    ['item_id' => $new_item_id, 'artifact' => $reassign_change_artifact],
+    "trackers_file", ['item_id' => $new_item_id, 'artifact' => $new_artifact],
     DB_AUTOQUERY_UPDATE, "item_id = ? AND artifact = ?", [$item_id, ARTIFACT]
   );
+  if ($result)
+    return;
+  $msg = sprintf (
+    _("Unable to duplicate an attached file (%s) from the\noriginal item."),
+    $row_attachment['filename']
+  );
+  fb ($msg, 1);
+}
 
-  if (!$result)
+function trackers_data_copy_cc_list ($item_id, $new_item_id, $new_artifact)
+{
+  $res = db_execute (
+    "SELECT * FROM " . ARTIFACT . "_cc WHERE bug_id = ?", [$item_id]
+  );
+  $table = "{$new_artifact}_cc";
+  while ($row = db_fetch_array ($res, DB_FETCH_ASSOC))
     {
+      unset ($row['bug_cc_id']);
+      $row['bug_id'] = $new_item_id;
+      if (db_autoexecute ($table, $row, DB_AUTOQUERY_INSERT))
+        continue;
       $msg = sprintf (
-       _("Unable to duplicate an attached file (%s) from the\n"
-         . "original item report information."),
-       $row_attachment['filename']
+        _("Unable to duplicate a CC address (%s) from the\noriginal item."),
+        $row['email']
       );
       fb ($msg, 1);
     }
+}
 
-  # Duplicate CC List.
-  $res_cc = db_execute (
-    "SELECT * FROM " . ARTIFACT . "_cc WHERE bug_id = ?",
-    [$item_id]
-  );
-  while ($row_cc = db_fetch_array ($res_cc))
-    {
-      $result = db_autoexecute (
-       "{$reassign_change_artifact}_cc",
-        [
-          'bug_id' => $new_item_id, 'email' => $row_cc['email'],
-          'added_by' => $row_cc['added_by'],
-          'comment' => $row_cc['comment'], 'date' => $row_cc['date']
-        ],
-        DB_AUTOQUERY_INSERT
-      );
+function trackers_data_reassign_string (
+  $new_artifact, $new_item_id
+)
+{
+  return 'THIS ITEM WAS REASSIGNED TO '
+    . strtoupper (utils_get_tracker_prefix ($new_artifact))
+    . " #$new_item_id\n";
+}
 
-      if (!$result)
-        {
-          $msg = sprintf (
-            _("Unable to duplicate a CC address (%s) from the\n"
-              . "original item report information."),
-            $row_cc['email']
-          );
-          fb ($msg, 1);
-        }
-    }
-
-  # Update data of the original to make sure people dont get confused.
-  # Close the original item.
-  $result = db_autoexecute (
-    ARTIFACT,
-    [
-      'status_id' => 3, 'close_date' => $now,
-      'summary' =>
-        "Reassigned to another tracker [was: {$row_data['summary']}]",
-      'details' => 'THIS ITEM WAS REASSIGNED TO '
-        . strtoupper (utils_get_tracker_prefix ($reassign_change_artifact))
-        . " #$new_item_id\n" . $row_data['details']
-    ],
-    DB_AUTOQUERY_UPDATE, "bug_id = ?", [$item_id]
-  );
-  trackers_data_add_history ('close_date', $now, $now, $item_id);
-
-  if ($result)
-      fb (_("Original item is now closed."));
-  else
-      fb (_("Unable to close the original item report."), 1);
-
-  # Finally put an extra comment so people dont get confused
-  # (it is not important, so run the sql without checks).
+function  trackers_data_reassign_close_add_history (
+  $item_id, $new_item_id, $new_artifact, $now
+)
+{
   db_autoexecute (
     ARTIFACT . "_history",
     [
       'bug_id' => $item_id, 'field_name' => 'details',
-      'old_value' => 'THIS ITEM WAS REASSIGNED TO '
-        . strtoupper (utils_get_tracker_prefix ($reassign_change_artifact))
-        . " #$new_item_id</p>\n"
-        . 'Please, do not post any new comments to this item.',
+      'old_value' => trackers_data_reassign_string (
+          $new_artifact, $new_item_id
+        )
+        . "<br />Please, do not post any new comments to this item.",
       'mod_by' => user_getid (), 'date' => $now, 'type' => 100
     ],
     DB_AUTOQUERY_INSERT
   );
+}
+
+function trackers_data_reassign_close_item (
+  $new_item_id, $new_artifact, $now, $data
+)
+{ $item_id = $data['bug_id']; $fields = [
+    'status_id' => 3, 'close_date' => $now,
+    'summary' => "Reassigned to another tracker [was: {$data['summary']}]",
+    'details' => trackers_data_reassign_string (
+       $new_artifact, $new_item_id
+     ) . $data['details']
+  ];
+  $result = db_autoexecute (
+    ARTIFACT, $fields,
+    DB_AUTOQUERY_UPDATE, "bug_id = ?", [$item_id]
+  );
+  trackers_data_add_history ('close_date', $now, $now, $item_id);
+  if ($result)
+    fb (_("Original item is now closed."));
+  else
+    fb (_("Unable to close the original item report."), 1);
+  trackers_data_reassign_close_add_history (
+    $item_id, $new_item_id, $new_artifact, $now
+  );
+}
+
+function trackers_data_reassign_followup ($new_item_id, $new_artifact)
+{
   $address = '';
   trackers_append_followup_notif_addresses (
-    $address, $new_item_id, true, $reassign_change_artifact
+    $address, $new_item_id, true, $new_artifact
   );
   trackers_mail_followup (
-    $new_item_id, $address, false, false, $reassign_change_artifact
+    $new_item_id, $address, false, false, $new_artifact
   );
+}
 
-  # If we get here, assume everything went properly.
+function trackers_data_pre_reassign ($item_id, $new_group, $new_artifact)
+{
+  $new_group_id =
+    trackers_data_reassign_get_new_group_id ($new_group, $new_artifact);
+  if ($new_group_id === null)
+    return false;
+
+  $now = time ();
+  $row_data = trackers_data_fetch_item ($item_id);
+
+  $new_item_id = trackers_data_copy_item (
+    $new_artifact, $new_group_id, $row_data, $now
+  );
+  return [$new_item_id, $new_group_id, $now, $row_data];
+}
+
+function trackers_data_reassign_item ($item_id, $new_group, $new_artifact)
+{
+  list ($new_item_id, $new_group_id, $now, $row_data) =
+    trackers_data_pre_reassign ($item_id, $new_group, $new_artifact);
+  if ($new_item_id === false)
+    return false;
+  trackers_data_reassign_add_history (
+    $item_id, $new_item_id, $new_group_id, $new_artifact
+  );
+  $comment = trackers_data_reassign_compose_comment ($item_id, $row_data);
+  trackers_data_reassign_add_comment (
+    $new_item_id, $new_artifact, $comment, $now
+  );
+  trackers_data_reassign_files ($item_id, $new_item_id, $new_artifact);
+  trackers_data_copy_cc_list ($item_id, $new_item_id, $new_artifact);
+  trackers_data_reassign_close_item (
+    $new_item_id, $new_artifact, $now, $row_data
+  );
+  trackers_data_reassign_followup ($new_item_id, $new_artifact);
   return true;
 }
 
