@@ -7,7 +7,7 @@
 # Copyright (C) 2002-2006 Yves Perrin <yves.perrin--cern.ch>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -52,7 +52,7 @@ extract (sane_import ('get',
   [
     'digits' =>
       [
-        'chunksz', 'offset', 'report_id',
+        'report_id',
         ['msort', 'sumORdet', 'advsrch', 'history_search', [0, 1]],
         ['spamscore', [1, null]],
         ['history_date_yearfd', [1900, null]],
@@ -63,7 +63,7 @@ extract (sane_import ('get',
     'strings' =>
       [
         ['func', ['default' => 'browse', 'digest']],
-        ['set', ['custom', 'my', 'open']],
+        ['set', ['custom', 'open']],
         ['history_event', ['modified', 'not modified']]
       ],
     'preg' =>
@@ -75,22 +75,13 @@ extract (sane_import ('get',
   ]
 ));
 
+html_nextprev_extract_params ();
 # Number of search criteria (boxes) displayed in one row.
 $fields_per_line = 5;
 
 # Avoid undesired user input.
 $browse_preamble = '';
 
-$default_chunksz = 50;
-
-# Number of bugs displayed on screen in one chunk.
-# Default 50.
-if (empty ($chunksz))
-  $chunksz = $default_chunksz;
-$chunksz = intval ($chunksz);
-if ($chunksz <= 0) # Catch values like "00" (non-empty, but intval is zero).
-  $chunksz = $default_chunksz;
-
 # Digest mode? Set the digest variable to one.
 $digest = $func == 'digest';
 
@@ -264,96 +255,103 @@ if (!trackers_report_init ($report_id))
     trackers_report_init ($report_id);
   }
 
-# See what type of bug set is requested (set is one of none,
-# 'my', 'open', 'custom').
-# - if no set is passed in, see if a preference was set ('custom' set).
-# - if no preference and logged in, then use 'my' set
-# - if no preference and not logged in, the use 'open' set
-#  (Prefs is a string of the form
-#  &amp;field1[]=value_id1&amp;field2[]=value_id2&amp;.... )
+function parse_history_field ($val)
+{
+  global $history_search, $history_field, $history_event, $history_date;
+  global $url_params;
+  $pref = explode ('>', $val);
+  if (count ($pref) != 4)
+    return;
+  list ($history_search, $history_field, $history_event, $history_date) = $pref;
+
+  # If not args in URL (means not after post) ...
+  # set $url_params['history'] explicitly since 'history'
+  # is not a tracker field and thus won't be set.
+  $url_params['history'][] = $val;
+}
+
+function apply_single_pref ($expr)
+{
+  global $max_rows, $url_params;
+
+  $fld_val = explode ('=', $expr, 2);
+  if (count ($fld_val) < 2)
+    return;
+  list ($field, $value) = $fld_val;
+  $field = str_replace ('[]', '', $field);
+  $global_fields = ['advsrch', 'msort', 'spamscore', 'report_id', 'sumORdet'];
+  if (in_array ($field, $global_fields))
+    $GLOBALS[$field] = $value;
+  elseif (in_array ($field, ['chunksz', 'max_rows']))
+    {
+      $max_rows = intval ($value);
+      if ($max_rows <= 0)
+        $max_rows = 50;
+    }
+  elseif ($field == 'history')
+    parse_history_field ($value);
+  else
+    $url_params[$field][] = $value;
+}
+
+function apply_user_prefs ($cust_pref)
+{
+  global $set;
+  if (!user_isloggedin ())
+    return;
+  $custom_pref = user_get_preference ($cust_pref);
+  if (!$custom_pref)
+    return;
+  $set = 'custom';
+  $pref_arr = explode ('&amp;', substr ($custom_pref, 5));
+  foreach ($pref_arr as $expr)
+    apply_single_pref ($expr);
+}
+
+function compile_pref_string ($url_params)
+{
+  # Use the list of fields built from the arguments and used by the project
+  # (the group_id parameter has been excluded).
+  # NB: Note that trackers_extract_field_list function did build and
+  # return date arguments (using _dayfd|monthfd|yearfd boxes) whether
+  # or not they were used by the group.
+
+  $ret = '';
+  foreach ($url_params as $field => $arr_val)
+    {
+      if (!is_array ($arr_val))
+        $arr_val = [$arr_val];
+      foreach ($arr_val as $value_id)
+        if (is_scalar ($value_id))
+          $ret .= "&amp;{$field}[]=$value_id";
+    }
+  $vars =
+    ['advsrch', 'msort', 'max_rows', 'spamscore', 'report_id', 'sumORdet'];
+  foreach ($vars as $v)
+    $ret .= "&amp;$v={$GLOBALS[$v]}";
+  return $ret;
+}
+
+# See what type of bug set is requested (set is one of none, 'open', 'custom').
+# * If no set is passed in, see if a preference was set ('custom' set).
+# * If no preference and not logged in, the use 'open' set.
+#  Preference is a string of the form
+#  &amp;field1[]=value_id1&amp;field2[]=value_id2...
 if (!$set)
   {
     $set = 'open';
-    if (user_isloggedin ())
-      {
-        $custom_pref = user_get_preference ($cust_pref);
-        if ($custom_pref)
-          {
-            $set = 'custom';
-            $pref_arr = explode ('&amp;', substr ($custom_pref, 5));
-            foreach ($pref_arr as $expr)
-              {
-                # Extract left and right parts of the assignment
-                # and remove the '[]' array symbol from the left part.
-                list ($field, $value_id) = explode ('=', $expr);
-                $field = str_replace ('[]', '', $field);
-                if ($field == 'advsrch' || $field == 'msort'
-                    || $field == 'spamscore' || $field == 'report_id'
-                    || $field == 'sumORdet'
-                )
-                  $$field = $value_id;
-                elseif ($field == 'chunksz')
-                  {
-                    $chunksz = intval ($value_id);
-                    if ($chunksz <= 0)
-                      $chunksz = $default_chunksz;
-                  }
-                elseif ($field == 'history')
-                  {
-                    $history = $value_id;
-                    $hist_pref = explode ('>', $history);
-                    $history_search = $hist_pref[0];
-                    $history_field = $hist_pref[1];
-                    $history_event = $hist_pref[2];
-                    $history_date = $hist_pref[3];
-
-                    # If not args in URL (means not after post) ...
-                    # set $url_params['history'] explicitly since 'history'
-                    # is not a tracker field and thus won't be set.
-                    $url_params['history'][] = "$history_search>"
-                      . "$history_field>$history_event>$history_date";
-                  }
-                else
-                  $url_params[$field][] = $value_id;
-              }
-          } # $custom_pref
-      } # user_isloggedin ()
+    apply_user_prefs ($cust_pref);
   } # !$set
 
 $user_id = user_getid ();
-if ($set == 'my')
+if ($set == 'custom')
   {
-    #  My bugs - backwards compat can be removed 9/10.
-    $url_params['status_id'][] = 1;
-    $url_params['assigned_to'][] = $user_id;
-  }
-elseif ($set == 'custom')
-  {
-    # Use the list of fields built from the arguments and used by the project
-    # (the group_id parameter has been excluded).
-    # NB: Note that trackers_extract_field_list function did build and
-    # return date arguments (using _dayfd|monthfd|yearfd boxes) whether
-    # or not they were tracker fields used by the project.
-
-    $pref_stg = '';
-    foreach ($url_params as $field => $arr_val)
-      {
-        if (!is_array ($arr_val))
-          $arr_val = [$arr_val];
-        foreach ($arr_val as $value_id)
-          if (is_scalar ($value_id))
-            $pref_stg .= "&amp;{$field}[]=$value_id";
-      }
-    $pref_stg .= "&amp;advsrch=$advsrch&amp;msort=$msort&amp;chunksz=$chunksz";
-    $pref_stg .= "&amp;spamscore=$spamscore&amp;report_id=$report_id";
-    $pref_stg .= "&amp;sumORdet=$sumORdet";
-
+    $pref_stg = compile_pref_string ($url_params);
     if ($pref_stg != user_get_preference ($cust_pref))
       user_set_preference ($cust_pref, $pref_stg);
   }
 else
   {
-    # We want to reset to all open items?
     # Force the status_id to open, set nothing else, trash the prefs.
     $url_params['status_id'][] = 1;
     user_unset_preference ($cust_pref);
@@ -387,15 +385,12 @@ while ($field = trackers_list_all_fields ())
 # Force the selection of privacy, we always want to be sure that no private
 # item title is provided to everybody.
 $full_field_list = $col_list = $width_list = $lbl_list = [];
-$select = [
-  'a.group_id', 'a.priority', 'a.privacy', 'a.status_id', 'a.submitted_by'
-];
+$select = ['a.group_id', 'a.priority', 'a.privacy', 'a.status_id'];
 
 $where = "WHERE\n    a.group_id = ?";
 $where_params = [$group_id];
 
-# Take into account the spamscore limit (always show
-# item posted by the logged-in user).
+# Take into account the spamscore, but show items posted by the logged-in user.
 $where .= " AND (a.spamscore < ?";
 $where_params[] = $spamscore;
 if ($user_id != 100)
@@ -405,18 +400,19 @@ if ($user_id != 100)
   }
 $where .= ")";
 
-# If the user asked for more than 150 items to be shown,
-# restrict arbitrarily to 150:
-# It would be too heavy on the database if this was done very frequently
-# and we already found some project giving direct links to 500 the browse
-# item page with 500 items shown by default.
-# Save the wanted number of chunksz, for later.
-$wanted_chunksz = $chunksz;
-if ($chunksz > 150 && !$digest)
-  $chunksz = 150;
+# If the user asked for more items to be shown than the configured limit,
+# restrict to the limit: it would be too heavy on the database if this was
+# done very frequently and we already found some group giving direct
+# links to 500 the browse item page with 500 items shown by default.
+# Save the wanted number of max_rows for later use.
+$wanted_max_rows = $max_rows;
+if (empty ($sys_max_items_per_page))
+  $sys_max_items_per_page = 150;
+if ($max_rows > $sys_max_items_per_page && !$digest)
+  $max_rows = $sys_max_items_per_page;
 
 $limit = "LIMIT ?, ?";
-$limit_params = [$offset, $chunksz];
+$limit_params = [$offset, $max_rows];
 
 # Prepare for summary and original submission as 'special' criteria.
 $summary_search = 0;
@@ -428,7 +424,10 @@ foreach ($url_params as $field => $val)
       continue;
     foreach ($val as $v)
       if (is_array ($v))
-        exit_error (_('Wrong parameter') . " $field");
+        {
+          $msg = utils_specialchars ("$field = " . error_print_r ($v));
+          exit_error (sprintf (_('Wrong parameter', $msg)));
+        }
   }
 
 foreach ($url_params as $field => $value_id)
@@ -671,10 +670,10 @@ while ($field = trackers_list_all_fields ('cmp_place_query'))
 if ($labels)
   $html_select .= "$labels</tr>\n$boxes</tr>\n";
 
-# Fill the relevant sql bit to be used later.
+# Fill the relevant SQL bit to be used later.
 # Sensible default case: order by item_id from the recent to the older
-# (only if not in multiple column sort, otherwise dont mess with it because
-# the first thing to be set will matters a lot).
+# (only if not in multiple column sort, otherwise don't mess with it because
+# the first thing to be set will matter a lot).
 if ($morder == '' && !$msort)
   $morder = "bug_id<";
 $order_by = null;
@@ -798,6 +797,17 @@ while ($field = trackers_list_all_fields ('cmp_place_result'))
     $where .= "\n    AND user_$field.user_id = a.$field";
   } # while ($field = trackers_list_all_fields ('cmp_place_result'))
 
+# The submitted_by column may be compared against the user id
+# for access to private items, so make sure it's present.
+$field = 'submitted_by';
+$submitted_by_column = "user_$field.user_name AS $field";
+if (!in_array ($submitted_by_column, $select))
+  {
+    $select[] = $submitted_by_column;
+    $froms[] = "user user_$field";
+    $where .= "\n    AND user_$field.user_id = a.$field";
+  }
+
 $art_h = "{$art}_history";
 
 $next_from = "$art a";
@@ -984,12 +994,10 @@ $form .= "</table>\n";
 if (($details_search == 1) && ($summary_search == 1))
   {
     $sum = rtrim (
-      trackers_field_label_display ("summary", $group_id, false, true),
-      ': '
+      trackers_field_label_display ("summary", $group_id, false, true), ': '
     );
     $det = rtrim (
-      trackers_field_label_display ("details", $group_id, false, true),
-       ': '
+      trackers_field_label_display ("details", $group_id, false, true), ': '
     );
     $conj =
       '<select title="' . _("logical operation to apply")
@@ -1087,7 +1095,7 @@ if ($history_search)
     . "&amp;history_date=$history_date";
 
 $form .= '<p class="smaller">';
-$form .= trackers_chunksz_control () . ' ';
+$form .= trackers_max_rows_control () . ' ';
 if ($is_trackeradmin)
   $form .=
     sprintf (
@@ -1096,20 +1104,14 @@ if ($is_trackeradmin)
         'size="3" maxlength="2" title="'
         . _("Spam level of items to hide") . '"')
     );
-if ($wanted_chunksz != $chunksz)
+if ($wanted_max_rows != $max_rows)
   {
-    # No use of ngettext as $chunksz will never be below 10, otherwise
-    # it would mean that Savane would be modified to never list more
-    # than 10 items at once, which is almost nothing.
-    $form .= ' <span class="warn">'
-      . sprintf (ngettext (
-          "Warning: only %s item can be shown at once, unless using "
-            . "Printer Version.",
-          "Warning: only %s items can be shown at once, unless using "
-            . "Printer Version.",
-           $chunksz), $chunksz
-        )
-      . '</span>';
+    $fmt = ngettext (
+      "Warning: only %s item can be shown at once.",
+      "Warning: only %s items can be shown at once.",
+      $max_rows
+    );
+    $form .= ' <span class="warn">' . sprintf ($fmt, $max_rows) . '</span>';
   }
 
 $form .= "</p>\n";
@@ -1152,17 +1154,17 @@ if ($totalrows > 0)
   }
 
 print html_show_displayoptions ($form, $form_opening, $form_submit);
-
-if ($digest)
-  print form_tag (['method' => 'get'])
-    . form_hidden (['group' => $group, 'func' => "digestselectfield"]);
-
 if ($totalrows > 0)
   {
-    show_item_list ($result_array, $offset, $totalrows, $col_list,
-      $lbl_list, $width_list, $url);
+    $nav_bar = html_nextprev_str ($url, $offset, $max_rows, $totalrows);
+    print "<div id='results'>$nav_bar</div>\n";
+    if ($digest)
+      print form_tag (['method' => 'get'])
+        . form_hidden (['group' => $group, 'func' => "digestselectfield"]);
+    show_item_list ($result_array, $col_list, $lbl_list, $width_list, $url);
     if ($digest)
       print form_footer (_("Proceed to Digest next step"));
+    print $nav_bar;
     show_priority_colors_key ();
   }
 else
