@@ -7,7 +7,7 @@
 # Copyright (C) 2002-2006 Yves Perrin <yves.perrin--cern.ch>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -171,11 +171,11 @@ $class = utils_get_priority_color (
 $caption = "<i>$item_link</i>: {$res_arr['summary']}";
 print html_h (1, $caption, ['class' => $class]);
 
-print form_header (
-  null, "post", 'enctype="multipart/form-data" name="item_form"'
+print form_tag (
+  ['enctype' => 'multipart/form-data', 'name' => 'item_form'], "?$item_id"
 );
 $hidden = [
-  "group_id" => $group_id, "item_id" => $item_id,
+  "group_id" => $group_id,
   'func' => $enable_comments? 'postmoditem': 'detailitem'
 ];
 print form_hidden ($hidden);
@@ -272,7 +272,7 @@ while ($field_name = trackers_list_all_fields ())
     # its values. If we already have a value, we are probably in
     # step 2 of a search.
 
-    # If nocache is set, we were explicetely asked to rely only
+    # If nocache is set, we were explicitely asked to rely only
     # on database content.
     if (!isset ($nocache))
       $nocache = false;
@@ -289,7 +289,8 @@ while ($field_name = trackers_list_all_fields ())
                 $$field_name = mktime (0, 0, 0, $mn, $dy, $yr);
               }
           }
-        $field_value = utils_specialchars ($$field_name);
+        if (isset ($$field_name))
+          $field_value = utils_specialchars ($$field_name);
       }
     list ($sz,) = trackers_data_get_display_size ($field_name);
     $label = trackers_field_label_display (
@@ -440,7 +441,7 @@ function print_comment_types ($group_id, $comment_type_id, $comment_types)
 function print_canned_box ($group_id, $canned, $size, $res_canned)
 {
   global $sys_home;
-  $group_admin = user_ismember ($group_id, 'A');
+  $group_admin = user_is_group_admin ($group_id);
   print "<span class='preinput'>";
   print html_label ('canned_response[]', _("Canned response:"))
     . "</span><br />\n";
@@ -593,19 +594,16 @@ print html_hidsubpart_header ("dependencies", _("Dependencies"));
 if ($is_trackeradmin)
   {
     print '<p class="noprint"><span class="preinput">';
-    if ($depends_search)
-      {
-        # Print a specific message if we are already at step 2 of filling
-        # a dependency.
-        print _("New search, in case the previous one was not satisfactory "
-                . "(to\nfill a dependency against):");
-      }
-    else
-      print _("Search an item (to fill a dependency against):");
-
+    print html_label ('depends_search',
+      $depends_search?
+        _("New search, in case the previous one was not satisfactory "
+          . "(to\nfill a dependency against):"):
+        _("Search an item (to fill a dependency against):")
+    );
     print "</span><br />\n&nbsp;&nbsp;&nbsp;"
-      . '<input type="text" title="' . _("Terms to look for")
-      . "\" name='depends_search' size='40' maxlength='255' /><br />\n";
+      . form_input ('text',  'depends_search', $depends_search,
+          "size='40' maxlength='255'"
+        ) . "<br />\n";
 
     $tracker_select =
     '&nbsp;&nbsp;&nbsp;<select title="' . _("Tracker to search in")
@@ -651,79 +649,10 @@ if ($is_trackeradmin)
     else
       print form_submit (_("Search"), "submit");
 
-    # Search results, if we are already at step 2.
-    if ($depends_search)
-      {
-        print "</p>\n<p><span class='preinput'>";
-        printf (
-          _("Please select a dependency to add in the result of your search\n"
-            . "of '%s' in the database:"),
-          utils_specialchars ($depends_search)
-        );
-        print '</span>';
-
-        $success = false;
-
-        # If we have less than 4 characters, to avoid giving lot of feedback
-        # and put an exit to the report, just consider the search as a failure.
-    if (strlen ($depends_search) > 3)
-          {
-            # Build the list of trackers to take account of.
-            if ($depends_search_only_artifact == "all")
-              $artifacts = ["support", "bugs", "task", "patch"];
-            else
-              $artifacts = [$depends_search_only_artifact];
-
-            # Actually search on each asked trackers.
-            foreach ($artifacts as $num => $tracker)
-              {
-                if ($depends_search_only_group == "notany")
-                  $GLOBALS['only_group_id'] = $group_id;
-
-                # Do not ask for all words,
-                $GLOBALS['exact'] = 0;
-
-                $result_search = search_run ($depends_search, $tracker, 0);
-                $success = db_numrows ($result_search) + $success;
-
-                # Print the result, if existing.
-                if (db_numrows ($result_search) == 0)
-                  continue;
-                while (list ($res_id, $res_summary, $res_date, $res_privacy,
-                             $res_submitter_id, $res_submitter_name,
-                             $res_group) = db_fetch_array ($result_search))
-                  {
-                    # Avoid item depending on itself.
-                    # Hide private items unless accessible.
-                    if (
-                      $res_privacy == 2 && !member_check_private (0, $res_group)
-                    )
-                      continue;
-                    if ($res_id != $item_id || $tracker != ARTIFACT)
-                      {
-                         # Right now only print id, summary and group.
-                         # We may change that depending on users feedback.
-                         print "<br />\n";
-                         $label = "$tracker #$res_id: $res_summary, "
-                           . _("group") . ' ' . group_getname ($res_group);
-                         print '&nbsp;&nbsp;&nbsp;'
-                           . form_checkbox (
-                               "dependent_on_{$tracker}[]", 0,
-                               ['value' => $res_id, 'label' => $label]
-                             );
-                      }
-                    }
-              } # foreach ($artifacts as $num => $tracker)
-          } # if (strlen ($depends_search) > 3)
-        if (!$success)
-          {
-            print "<br />\n<span class='warn'>";
-            print
-              _("None found. Please note that only search words of more than\n"
-                . "three characters are valid.");
-            print '</span>';
-          }
-      } # if ($depends_search)
+    trackers_search_dependencies (
+      $depends_search, $depends_search_only_artifact,
+      $depends_search_only_group
+    );
     print "</p>\n";
   } # if ($is_trackeradmin)
 print show_item_dependency ($item_id);
@@ -890,8 +819,11 @@ if ($is_manager && ARTIFACT != "cookbook")
 
         $res = search_run ($reassign_change_group_search, "soft", 0);
         $list_is_empty = true;
-        while (list ($grp_name, $ug_name, $grp_id) = db_fetch_array ($res))
+        while ($row = db_fetch_array ($res))
           {
+            $grp_id = $row['group_id'];
+            $ug_name = $row['unix_group_name'];
+            $grp_name = $row['group_name'];
             if ($grp_id == $group_id) # Don't reassign to itself.
               continue;
             $list_is_empty = false;
