@@ -6,8 +6,8 @@
 # Copyright (C) 2002-2006 Tobias Toedter <t.toedter--gmx.net>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
-# Copyright (C) 2022, 2024 Bob Proulx
+# Copyright (C) 2015-2020, 2022, 2024 Bob Proulx
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -237,13 +237,14 @@ function utils_cutstring ($string, $length = 35)
 function utils_utils_strftime ($timestamp, $format)
 {
   $env['LC_ALL'] = setlocale (LC_TIME, 0);
+  $env['TZ'] = getenv ('TZ');
   $cmd = "date +$format -d @$timestamp";
   utils_run_proc ($cmd, $output, $error, ['env' => $env]);
   $output = substr ($output, 0, -1); # Drop trailing "\n".
   return $output;
 }
 
-if (function_exists ("strftime"))
+if (!empty ($sys_use_strftime) && function_exists ("strftime"))
   {
     # Used until strftime is dropped from PHP.
     function utils_strftime ($timestamp, $format)
@@ -254,14 +255,14 @@ if (function_exists ("strftime"))
       return $ret;
     }
   }
-else # !function_exists ("strftime")
+else # !(!empty ($sys_use_strftime) && function_exists ("strftime"))
   {
-    # Fallback for the PHP versions (> 8.1) when no strftime is provided.
+    # Fallback for the PHP versions (> 8.1) when no usable strftime is provided.
     function utils_strftime ($timestamp, $format)
     {
       return utils_utils_strftime ($timestamp, $format);
     }
-  } # !function_exists ("strftime")
+  } # !(!empty ($sys_use_strftime) && function_exists ("strftime"))
 
 # Return a formatted date for a unix timestamp.
 #
@@ -473,6 +474,11 @@ function utils_get_tracker_list ()
   return ['bugs', 'support', 'task', 'patch', 'cookbook'];
 }
 
+function utils_get_dependable_trackers ()
+{
+  return array_diff (utils_get_tracker_list (), ['cookbook']);
+}
+
 function utils_get_tracker_icon ($tracker)
 {
   if ($tracker == "bugs")
@@ -1010,7 +1016,8 @@ function utils_disable_cache ()
 function utils_public_file_url ($file)
 {
   global $sys_home;
-  return "{$sys_home}file/{$file['name']}?file_id={$file['id']}";
+  $name = urlencode ($file['name']);
+  return "{$sys_home}file/$name?file_id={$file['id']}";
 }
 
 # Find an item in a tracker, fetch the fields listed in $fields
@@ -1277,6 +1284,7 @@ function utils_get_assign_idNumber_params ($type)
 
 # Try to assign a new idNumber to the database entity $id (in the context
 # of $type).  Return the new idNumber in case of success, else return zero.
+# When the entity already has an idNumber, return its current idNumber.
 function utils_try_assign_idNumber ($id, $type)
 {
   $params = utils_get_assign_idNumber_params ($type);
@@ -1294,8 +1302,8 @@ function utils_try_assign_idNumber ($id, $type)
   return $idN;
 }
 
-# Assign a new idNumber depending on $type ('user' or 'group'),
-# return its value on success and zero on failure.
+# Assign a new idNumber depending on $type ('user' or 'group') unless
+# it has already been assigned, return its value on success and zero on failure.
 function utils_assign_idNumber ($id, $type = 'user')
 {
   for ($i = 0; $i < 0x11; $i++)
