@@ -7,7 +7,7 @@
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
 # Copyright (C) 2008-2017, 2020 Karl Berry
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -170,19 +170,6 @@ function update_gpgkey ()
   return $success;
 }
 
-function extract_email_newvalue ()
-{
-  $vals = sane_import ('request',
-    [
-      'preg' =>
-        [['newvalue', '/^[a-zA-Z0-9_.+-]+@(([a-zA-Z0-9-])+\.)+[a-zA-Z0-9]+$/']]
-    ]
-  );
-  if (empty ($vals['newvalue']))
-    return '';
-  return preg_replace ('/\s/', '', $vals['newvalue']);
-}
-
 function confirm_hash_url ($confirm_hash, $func = 'email')
 {
   global $sys_https_host, $sys_home, $sys_default_domain;
@@ -264,10 +251,22 @@ function report_step0_result ($fail, $newval)
   fb ($msg, $fail);
 }
 
+function email_is_unchanged ($val)
+{
+  if ($val !== user_get_email (user_getid ()))
+    return 0;
+  fb (sprintf (_('*%s* is already your registered email.'), $val), 1);
+  return 1;
+}
+
 function update_email_step0 ()
 {
   global $item;
-  $val = extract_email_newvalue ();
+  $val = '';
+  if (array_key_exists ('newvalue', $_REQUEST))
+    $val = $_REQUEST['newvalue'];
+  if (email_is_unchanged ($val))
+    return;
   if (!account_emailvalid ($val))
     return;
   $confirm_hash = account_generate_confirm_hash ($item, ['email_new' => $val]);
