@@ -6,7 +6,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -299,7 +299,7 @@ if ($words)
 
         while ($val = db_fetch_array ($result_search))
           {
-            if (user_check_ismember ($row_user['user_id'], $val['group_id']))
+            if (user_ismember ($row_user['user_id'], $val['group_id']))
               {
                 print "+ {$val['group_name']} ";
                 print _('(already a member)') . "<br />\n";
