@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -46,7 +46,7 @@ extract (sane_import ('get', ['digits' => 'user_id']));
 
 exit_if_missing ('user_id');
 if (!user_exists ($user_id))
-  exit_error (sprintf (_("User #%s not found."), $user_id));
+  exit_user_not_found ($user_id);
 $user_name = user_getname ($user_id);
 $key = user_get_gpg_key ($user_id);
 if (empty ($key))
