@@ -6,7 +6,7 @@
 # Copyright (C) 2008 Aleix Conchillo Flaque
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -47,7 +47,7 @@ require_once ('../include/vars.php');
 require_once ('../include/form.php');
 require_directory ("project");
 
-session_require (['group' => '1','admin_flags' => 'A']);
+session_require (['group' => SESSION_ADMIN_GROUP]);
 
 # We don't internationalize messages in this file because they are
 # for Savannah admins who use English.
