@@ -5,7 +5,7 @@
 # Copyright (C) 2000-2006 Mathieu Roy <yeupou--gnu.org>
 # Copyright (C) 2014, 2016, 2017 Assaf Gordon
 # Copyright (C) 2001-2011, 2013, 2017 Sylvain Beucler
-# Copyright (C) 2013, 2014, 2017-2024 Ineiev
+# Copyright (C) 2013, 2014, 2017-2025 Ineiev
 #
 # This file is part of Savane.
 #
@@ -44,8 +44,7 @@
 $testconfig_php = true;
 require_once ("include/ac_config.php");
 $sys_file_domain = '';
-$sys_linguas = "en:es";
-foreach (['i18n', 'database', 'mailman', 'savane-git'] as $inc)
+foreach (['database', 'mailman', 'savane-git'] as $inc)
   require_once ("include/$inc.php");
 
 function return_bytes ($v)
@@ -253,30 +252,43 @@ function test_gpg ()
   test_sys_gpg_key ();
 }
 
-function test_cgitrepos ()
+function test_cgitrepos_presence ()
 {
   if (!isset ($GLOBALS['sys_etc_dir']))
-    return '<strong>no $sys_etc_dir set</strong>';
+    return ['<strong>no $sys_etc_dir set</strong>', null];
   if (!file_exists ($GLOBALS['sys_etc_dir']))
-    return '<strong>no $sys_etc_dir directory exists</strong>';
+    return ['<strong>no $sys_etc_dir directory exists</strong>', null];
   $fname = $GLOBALS['sys_etc_dir'] . '/cgitrepos';
   if (!file_exists ($fname))
-    return '<strong>no cgitrepos file exists in $sys_etc_dir</strong>';
+    return ['<strong>no cgitrepos file exists in $sys_etc_dir</strong>', null];
   if (!is_readable ($fname))
-    return '<strong>cgitrepos in $sys_etc_dir is not readable</strong>';
+    return ['<strong>cgitrepos in $sys_etc_dir is not readable</strong>', null];
+  return [null, $fname];
+}
+
+function test_cgitrepos_time ($fname)
+{
   $mtime = time () - filemtime ($fname);
-  if ($mtime > 3600)
-    {
-      $ret = ('<strong>cgitrepos has not been updated for ');
-      if ($mtime < 100)
-        $ret .= sprintf ('%.0f minutes</strong>', $mtime / 60);
-      else if ($mtime < 24 * 3600)
-        $ret .= sprintf ('%.0f hours</strong>',  $mtime / 3600);
-      else
-        $ret .= sprintf ('%.1f days</strong>',  $mtime / 24. / 3600);
-      return $ret;
-    }
-  return 'OK';
+  if ($mtime <= 3600)
+    return null;
+  $ret = ('<strong>cgitrepos has not been updated for ');
+  if ($mtime < 100)
+    return $ret . sprintf ('%.0f minutes</strong>', $mtime / 60);
+  if ($mtime < 24 * 3600)
+    return $ret . sprintf ('%.0f hours</strong>',  $mtime / 3600);
+  return $ret . sprintf ('%.1f days</strong>',  $mtime / 24. / 3600);
+}
+
+function test_cgitrepos ()
+{
+  $disclaimer = 'Note that cgitrepos is an obsolete way to get git info.';
+  list ($ret, $path) = test_cgitrepos_presence ();
+  if ($ret !== null)
+    return "$ret<br />$disclaimer";
+  $ret = test_cgitrepos_time ($path);
+  if ($ret === null)
+    return 'OK';
+  return "$ret<br />$disclaimer";
 }
 
 function sys_vcs_dir_not_set ()
@@ -614,16 +626,72 @@ $phpfunctions =
  ];
 $page .= list_facilities ('function_exists', $phpfunctions);
 
+function test_language ($lang)
+{
+  global $locale_list;
+  if (!array_key_exists ($lang, $locale_list))
+    return "Locale for $lang is not defined.";
+  $loc = $locale_list[$lang];
+  $err = i18n_setup ($loc);
+  if (!empty ($err))
+    {
+      $ret = "<b>Fail:</b>\n<ul>\n";
+      foreach ($err as $e)
+        $ret .= "\n  <li>$e</li>\n";
+      return "$ret</ul>\n";
+    }
+  foreach (['Any', 'Apply'] as $str)
+    {
+      $res = gettext ($str);
+      if (($res == $str  && $lang == 'en') || ($res != $str && $lang != 'en'))
+        return "$str => $res";
+     }
+  $ret = "<b>Fail.</b> Check <code>locale -a</code> output,\n"
+    . "be sure to install language-pack-* in Trisquel";
+  return $ret;
+}
+
+function test_disabled_languages ($loc_list)
+{
+  global $sys_linguas;
+  $linguas = $sys_linguas;
+  $ret = "<p>Other defined languages: ";
+  if (empty ($loc_list))
+    return "{$ret}none.</p>\n";
+  $ret .= join (", ", array_keys ($loc_list)) . ".</p>\n";
+  $defs = [];
+  foreach ($loc_list as $code => $locale_names)
+    {
+      $sys_linguas .= ":$code";
+      register_language ($code, $locale_names[0], $locale_names[1]);
+      $defs[$code] = test_language ($code);
+    }
+  $sys_linguas = $linguas;
+  return $ret . html_dl ($defs);
+}
+
+function get_locales ()
+{
+  $res = utils_run_proc ('locale -a', $out, $err);
+  if ($res)
+    return "<p>Can't get locales defined: </p>\n<pre>$err</pre>\n";
+  $out = str_replace ("\n", ', ', substr ($out, 0, -1));
+  return "<p>Locales defined: $out.</p>\n";
+}
+
 function test_i18n ()
 {
-  i18n_setup ("es_ES.UTF-8");
-  $str = 'Any';
-  $res = gettext ($str);
-  if ($res == $str)
-    $ret = "<b>Fail.</b> Check <code>locale -a</code> output,\n"
-      . "be sure to install language-pack-* in Trisquel";
-  else
-    $ret = "$str => $res";
+  global $sys_linguas, $languages_available;
+  $loc_list = $languages_available;
+  $ret = get_locales () . "<p>sys_linguas: $sys_linguas</p>\n";
+  $defs = [];
+  foreach (explode (':', $sys_linguas) as $lang)
+    {
+      $defs[$lang] = test_language ($lang);
+      if (array_key_exists ($lang, $loc_list))
+        unset ($loc_list[$lang]);
+    }
+  $ret .= html_dl ($defs) . test_disabled_languages ($loc_list);
   i18n_setup ("en_US.UTF-8");
   return $ret;
 }
@@ -765,17 +833,16 @@ function try_db_connect ()
 }
 function mysql_params_to_test ()
 {
-  $wrong_modes = ['STRICT_TRANS_TABLES'];
-  $mysql_params = [
+  $wrong_modes = []; # No unsupported modes these days.
+  $comment = null;
+  if (!empty ($wrong_modes))
+    $comment = "<em>This should</em> <strong>not</strong> <em>include</em> "
+      . "<code>" . join (',', $wrong_modes) . "</code><em>.</em>";
+  return [
     '@@GLOBAL.version' => [null],
     '@@GLOBAL.sql_mode' => [null, $wrong_modes],
-    '@@SESSION.sql_mode' => [
-      "<em>This should</em> <strong>not</strong> <em>include</em> "
-        . "<code>" . join(',', $wrong_modes) . "</code><em>.</em>",
-      $wrong_modes
-    ]
+    '@@SESSION.sql_mode' => [$comment, $wrong_modes]
   ];
-  return $mysql_params;
 }
 
 function test_mysql_params ()
@@ -787,11 +854,8 @@ function test_mysql_params ()
       $result = db_execute ("SELECT $param");
       $value = db_result ($result, 0, $param);
       if (isset ($comment[1]))
-        {
-          $vals = $comment[1];
-          foreach ($vals as $i => $v)
-            $value = str_replace ($v, "<strong>$v</strong>", $value);
-        }
+        foreach ($comment[1] as $flag)
+          $value = str_replace ($flag, "<strong>$flag</strong>", $value);
       $value = "'$value'";
       if ($comment[0] !== null)
         $value .= "<br />\n{$comment[0]}";
@@ -855,9 +919,10 @@ function output_sysvars ()
     'min_gidNumber', 'min_uidNumber', 'group_file', 'passwd_file',
     'passwd_common_gid', 'passwd_home_dir', 'passwd_user_shell',
     'pw_rounds', 'reply_to',
-    'themedefault', 'unix_group_name', 'upload_max',
+    'themedefault', 'unix_group_name', 'upload_max', 'use_strftime',
     'watch_anon_posts', 'new_user_watch_days',
-    'mailman_wrapper', 'savane_cgit', 'group_file', '!debug_footer'
+    'mailman_wrapper', 'savane_cgit', 'group_file', 'max_items_per_page',
+    '!debug_footer'
   ];
   $defs = [];
   foreach ($variables as $tag)
@@ -953,7 +1018,10 @@ function test_sysconfigs ()
 $page .= html_h (2, "Configured settings");
 
 if (is_readable ($sys_conf_file))
-  test_sysconfigs ();
+  {
+    require_once ("include/i18n.php");
+    test_sysconfigs ();
+  }
 else
   print "$page\nSince $sys_conf_file does not exist or is not readable, "
     . "this part cannot be checked.";
